#ifndef  F_CPU
#define F_CPU 16000000UL
#endif

#include <avr/io.h>
#include <util/twi.h>

#include "i2c_master.h"

#define F_SCL 100000UL // SCL frequency
#define Prescaler 1
#define TWBR_val ((((F_CPU / F_SCL) / Prescaler) - 16 ) / 2)

void I2C_init(void)
{
	TWBR = (uint8_t)TWBR_val;
}

uint8_t I2C_start(void)
{
	TWCR = 0;												// reset TWI control register

	TODO_1												// transmit START condition 
	while( !(TWCR & (1<<TWINT)) ); 						// wait for end of transmission
	
	
	if((TWSR & 0xF8) != TW_START){ return 1; }			// check if the start condition was successfully transmitted

	return 0;
}

uint8_t I2C_write(uint8_t data)
{
	TWDR = data;											// load data into data register

	TODO_2												// start transmission of data
	while( !(TWCR & (1<<TWINT)) );						// wait for end of transmission
	
	if( (TWSR & 0xF8) != TW_MT_DATA_ACK ){ return 1; }
	
	return 0;
}

uint8_t I2C_read_ack(void)
{
	TODO_3												// start TWI module and acknowledge data after reception
	
	while( !(TWCR & (1<<TWINT)) );						// wait for end of transmission
	
	TODO_4												// return received data from TWDR
}

uint8_t I2C_read_nack(void)
{
	TODO_5												// start receiving without acknowledging reception
	
	while( !(TWCR & (1<<TWINT)) );						// wait for end of transmission
	
	TODO_6												// return received data from TWDR
}

void I2C_stop(void)
{
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);	// transmit STOP condition
}

